#!/bin/sh
# (rpm)trigger script for okas
# $Id: triggerin.sh,v 1.10 2004/10/12 10:13:52 glen Exp $
. /usr/lib/okas/functions
umask 002

case "$1" in
vim)
	vim=`which vim 2>/dev/null`
	[ "$vim" ] || exit 0

	vimrc=`vim --version 2>&1 | awk '/system vimrc/ {print $NF}' | xargs`

	if [ "$vimrc" ]; then
		PatchFile "$vimrc" \
			'"?.*so.*vim/okas\.vim' \
			's|^"\?.*so.*vim/okas\.vim.*$|so /usr/share/okas/vim/okas.vim|g' \
			'so /usr/share/okas/vim/okas.vim'
	fi
;;


bashrc)
	# bashrc
	if [ -f /etc/bash.bashrc ]; then # SuSE
		bashrc=/etc/bash.bashrc.local

	elif [ -f /etc/bashrc.local ]; then # we
		bashrc=/etc/bashrc.local

	else
		bashrc=/etc/bashrc

	fi

	PatchFile $bashrc \
		'#?.*okas/bashrc' \
		's|^#\?.*okas/bashrc.*$|test "$PS1" -a "$BASH_VERSION" \&\& . /usr/share/okas/bashrc|g' \
		'test "$PS1" -a "$BASH_VERSION" && . /usr/share/okas/bashrc'
;;


hosts)
	host=`hostname -f`
	case $host in
	*.delfi.* | *.online.ee)
		if /sbin/route -n | grep -q '^192.168.5.' ; then
			localip='/usr/share/okas/hosts.delfi.ee'
		fi
		# insert or replace delfi hostnames
		PatchFile /etc/hosts \
			'delfi.hosts' \
			"
			/^# START delfi.hosts/,/^# END delfi.hosts/ {
				/^# END/b
				/^# START/ {
					s/^.*\$/# START delfi.hosts (from okas package) CHANGES DONE TO OKAS SECTION WILL BE LOST AFTER NEXT UPGRADE SO YOU DO YOUR MODIFICATIONS BEFORE OKAS SECTION./
					p
					${localip:+r $localip}
					r /usr/share/okas/hosts.delfi
				}
				d
			}
		" \
		"
# START delfi.hosts from (okas package)
`cat $localip /usr/share/okas/hosts.delfi 2>/dev/null`
# END delfi.hosts

"
	;;
	esac
;;


sysctl.conf)
	[ -f /etc/sysctl.conf ] || exit 0

	[ "`uname -s`" = Linux ] || exit 0

	# hack /etc/sysctl.conf
	for key in `sed -e '/^#/d;/^$/d;s/ .*//' /usr/share/okas/sysctl.conf`; do
		grep -q "^[# ]*$key" /etc/sysctl.conf && continue
		sed -e '/./{H;$!d;}' -e "x;/$key/!d" /usr/share/okas/sysctl.conf >> /etc/sysctl.conf
	done

;;


rpm.macros)
	# RPM tune
	if [ -d /var/lib/rpm ]; then
		[ ! -d /etc/rpm ] && mkdir -m 755 /etc/rpm
		[ ! -f /etc/rpm/macros ] && touch /etc/rpm/macros

		# enable _install_langs only when it was commented out. don't force
		PatchFile /etc/rpm/macros \
			'^#?%_install_langs.*$' \
			's|^#.*\(%_install_langs\)|\1|g' \
			'%_install_langs et_EE:lv_LV:lt_LT:ru_RU:en_US'
	fi
;;


XTerm|XFree86)
	# XTerm App-Defaults
	[ -f /usr/X11R6/lib/X11/app-defaults/XTerm ] || exit 0
	PatchFile \
		/usr/X11R6/lib/X11/app-defaults/XTerm \
		'!?.*#include.*okas/XTerm.ad"' \
		's|^!\?.*#include.*okas/XTerm.ad".*$|#include "/usr/share/okas/XTerm.ad"|g' \
		'#include "/usr/share/okas/XTerm.ad"'

;;

sshd_config)
	[ -f /etc/ssh/sshd_config ] || exit 0

	old=`mktemp ${TMPDIR:-/tmp}/sshdXXXXXX`
	cp /etc/ssh/sshd_config $old
	dosed /etc/ssh/sshd_config \
		"
		/^#Protocol /s:.*:Protocol 2:
		/^#PermitRootLogin /s:.*:PermitRootLogin no:
		"

	if ! cmp $old /etc/ssh/sshd_config &>/dev/null; then
		echo >&2 "WARNING:"
		echo >&2 "WARNING: !!! sshd_config was modified to change PermitRootLogin or Protocol value."
		echo >&2 "WARNING: !!! You have explicitly enable them back if you need root logins or ssh protocol version 1."
		echo >&2 "WARNING:"

		if [ -f /var/lock/subsys/sshd ]; then
			/etc/rc.d/init.d/sshd restart
		fi
	fi
	rm -f $old
;;
esac
